# -*- coding: utf-8 -*-
import re
from a3json_struct.errors import ValidationError


class RegexValidator:
    _pattern = ""
    _flags = 0
    _inverse_match = False
    _error_message = "Enter a valid value."

    def __init__(
        self,
        pattern: str | None = None,
        flags: int | None = None,
        inverse_match: bool | None = None,
        error_message: str | None = None,
    ):
        if pattern is not None:
            self._pattern = pattern
        if flags is not None:
            self._flags = flags
        if inverse_match is not None:
            self._inverse_match = inverse_match
        if error_message is not None:
            self._error_message = error_message

        self._compiled_regex = re.compile(self._pattern, self._flags)

    def __call__(self, value: str):
        regex_matches = self._compiled_regex.search(value)
        is_invalid = regex_matches is None
        if self._inverse_match:
            is_invalid = not is_invalid

        if is_invalid:
            raise ValidationError(self._error_message)



from typing import Any, Tuple
import datetime

from a3json_struct.errors import ValidationError
from .abstract_field import AbstractField
from .utils import JsonType, OpenAPIFormat


INT_DATE_FORMAT = "%Y%m%d"


class IntDateField(AbstractField):
    def _cast_to_python(self, value: Any) -> datetime.date:
        if isinstance(value, datetime.date):
            return value

        elif isinstance(value, int):
            try:
                return datetime.datetime.strptime(str(value), INT_DATE_FORMAT)
            except ValueError:
                raise ValidationError(
                    f'Value "{value}" must has the correct format ({INT_DATE_FORMAT})，and must be a valid date.'
                )
        else:
            try:
                return datetime.date.fromisoformat(str(value))
            except ValueError:
                raise ValidationError(
                    f'Value "{value}" must has the correct format (YYYY-MM-DD)，and must be a valid date.'
                )

    def _cast_to_json(self, cleaned_value: datetime.date) -> int:
        return int(cleaned_value.strftime("%Y%m%d"))

    def _get_json_type_and_openapi_format(self) -> Tuple[str, str]:
        return JsonType.Number, OpenAPIFormat.Date



from typing import Any, Set, Tuple
from a3json_struct.errors import ValidationError
from a3json_struct import validators

from .abstract_field import AbstractField
from .utils import JsonType, OpenAPIFormat, set_nonempty_kv


class IntegerField(AbstractField):
    def __init__(
        self,
        min_value: int | None = None,
        max_value: int | None = None,
        choices: Set[int] | None = None,
        *args,
        **kwargs,
    ):
        super().__init__(*args, **kwargs)

        self._min_value = min_value
        self._max_value = max_value
        self._choices = choices

        if self._min_value is not None:
            self._validators.append(validators.MinValueValidator(self._min_value))
        if self._max_value is not None:
            self._validators.append(validators.MaxValueValidator(self._max_value))

    def _validate(self, value: Any) -> Any:
        value = super()._validate(value)
        if self._choices is not None and value is not None and value not in self._choices:
            raise ValidationError(f'Value "{value}" is not a valid choice.')
        return value

    def _cast_to_python(self, value: Any) -> int:
        if isinstance(value, int):
            return value

        try:
            return int(value)
        except (TypeError, ValueError):
            raise ValidationError(f'Value "{value}" is not a valid integer number.')

    def _cast_to_json(self, cleaned_value: int) -> int:
        return cleaned_value

    def _get_json_type_and_openapi_format(self) -> Tuple[str, str]:
        return JsonType.Number, OpenAPIFormat.Long

    def generate_openapi_object(self) -> dict:
        od = super().generate_openapi_object()

        set_nonempty_kv(od, "minimum", self._min_value)
        set_nonempty_kv(od, "maximum", self._max_value)

        if self._choices is not None:
            od["enum"] = list(self._choices)

        return od

    def generate_meta_object(self) -> dict:
        od = super().generate_meta_object()

        od["min_value"] = self._min_value
        od["max_value"] = self._max_value
        od["choices"] = list(self._choices) if self._choices is not None else None

        return od



from typing import Any, Tuple
from decimal import Decimal, DecimalException

from a3json_struct.errors import ValidationError
from a3json_struct import validators
from .abstract_field import AbstractField
from .utils import JsonType, OpenAPIFormat


class DecimalField(AbstractField):
    def __init__(self, min_value: Decimal | None = None, max_value: Decimal | None = None, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self._min_value = min_value
        self._max_value = max_value

        if self._min_value is not None:
            self._validators.append(validators.MinValueValidator(self._min_value))
        if self._max_value is not None:
            self._validators.append(validators.MaxValueValidator(self._max_value))

    def _cast_to_python(self, value: Any) -> Decimal:
        if isinstance(value, Decimal):
            return value

        try:
            return Decimal(str(value))
        except DecimalException:
            raise ValidationError(f'Value "{value}" is not a valid decimal number.')

    def _cast_to_json(self, cleaned_value: Decimal) -> str:
        return str(cleaned_value)

    def _get_json_type_and_openapi_format(self) -> Tuple[str, str]:
        return JsonType.String, OpenAPIFormat.Decimal

    def generate_meta_object(self) -> dict:
        od = super().generate_meta_object()

        od["min_value"] = self._min_value
        od["max_value"] = self._max_value

        return od



# -*- coding: utf-8 -*-
from typing import Any


def set_nonempty_kv(d: dict, key: str, v: Any):
    if v not in [None, ""]:
        d[key] = v


class JsonType:
    Boolean = "boolean"
    String = "string"
    Number = "number"
    Array = "array"
    Object = "object"


class OpenAPIFormat:
    Boolean = "boolean"
    String = "string"
    Date = "date"
    Datetime = "datetime"
    Decimal = "decimal"
    Object = "object"
    Float = "float"
    Long = "long"
    Array = "array"



from .abstract_limit_validator import AbstractIntLimitValidator, Number


class MaxValueValidator(AbstractIntLimitValidator):
    def _clean(self, value: Number) -> Number:
        return value

    def _check_if_valid(self, limit_value: Number, cleaned_value: Number) -> bool:
        return cleaned_value <= limit_value

    def _get_error_message(self, limit_value: Number) -> str:
        return f"Ensure this value is less than or equal to {limit_value}."



from typing import Any, Tuple

from a3json_struct.errors import ValidationError
from .abstract_field import AbstractField
from .utils import JsonType, OpenAPIFormat


class DictField(AbstractField):
    def _cast_to_python(self, value: Any) -> dict:
        if isinstance(value, dict):
            return value

        raise ValidationError(f'Value "{value}" is not a valid dict.')

    def _cast_to_json(self, cleaned_value: dict) -> dict:
        return cleaned_value

    def _get_json_type_and_openapi_format(self) -> Tuple[str, str]:
        return JsonType.Object, OpenAPIFormat.Object



from typing import Any

from .abstract_limit_validator import AbstractIntLimitValidator, Number


class MinLengthValidator(AbstractIntLimitValidator):
    def _get_error_message(self, limit_value: Number) -> str:
        return f"Ensure this value has at least {limit_value} elements."

    def _clean(self, value: Any) -> Number:
        return len(value)

    def _check_if_valid(self, limit_value: Number, cleaned_value: Number) -> bool:
        return cleaned_value >= limit_value



import abc
from typing import Any, Union
from decimal import Decimal

from a3json_struct.errors import ValidationError


Number = Union[float, int, Decimal]


class AbstractIntLimitValidator(abc.ABC):
    def __init__(self, limit_value: Number):
        self._limit_value = limit_value

    @abc.abstractmethod
    def _clean(self, value: Any) -> Number:
        raise NotImplementedError()

    @abc.abstractmethod
    def _check_if_valid(self, limit_value: Number, cleaned_value: Number) -> bool:
        raise NotImplementedError()

    @abc.abstractmethod
    def _get_error_message(self, limit_value: Number) -> str:
        raise NotImplementedError()

    def __call__(self, value: Any):
        cleaned_value = self._clean(value)
        if not self._check_if_valid(self._limit_value, cleaned_value):
            raise ValidationError(self._get_error_message(self._limit_value))



# -*- coding: utf-8 -*-
from typing import Any, Type, TYPE_CHECKING, Tuple
from a3json_struct.errors import ValidationError

from .abstract_field import AbstractField
from .utils import JsonType, OpenAPIFormat

if TYPE_CHECKING:
    from a3json_struct.json_struct import JsonStruct


class ObjectField(AbstractField):
    def __init__(self, obj_kls: Type["JsonStruct"], *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._obj_kls = obj_kls

    def _cast_to_python(self, value: Any) -> "JsonStruct":
        if isinstance(value, dict):
            obj_instance = self._obj_kls(**value)
        elif isinstance(value, self._obj_kls):
            obj_instance = value
        else:
            raise ValidationError(f'Value must be a dict or type {self._obj_kls.__name__}, not "{type(value)}".')

        obj_instance.full_clean()
        return obj_instance

    def _cast_to_json(self, cleaned_value: "JsonStruct") -> dict:
        return cleaned_value.to_json()

    def _cast_to_bson(self, cleaned_value: "JsonStruct") -> dict:
        return cleaned_value.to_bson()

    def _get_json_type_and_openapi_format(self) -> Tuple[str, str]:
        return JsonType.Object, OpenAPIFormat.Object

    def generate_openapi_object(self) -> dict:
        od = super().generate_openapi_object()

        sub_od = self._obj_kls.generate_openapi_schema()
        od.update(sub_od)

        return od

    def generate_meta_object(self) -> dict:
        od = super().generate_meta_object()

        od["obj_kls_meta"] = self._obj_kls.generate_meta_schema()
        od["obj_kls_name"] = self._obj_kls.__name__

        return od



# -*- coding: utf-8 -*-
from typing import Any, List, Iterable, Tuple
from a3json_struct.errors import ValidationError
from a3json_struct import validators

from .abstract_field import AbstractField
from .utils import JsonType, OpenAPIFormat, set_nonempty_kv


class ListField(AbstractField):
    def __init__(
        self,
        element_field: AbstractField,
        unique: bool = False,
        min_length: int | None = None,
        max_length: int | None = None,
        *args,
        **kwargs,
    ):
        super().__init__(*args, **kwargs)

        self._element_field = element_field
        self._unique = unique

        self._min_length = min_length
        self._max_length = max_length
        if self._min_length is not None:
            self._validators.append(validators.MinLengthValidator(self._min_length))
        if self._max_length is not None:
            self._validators.append(validators.MaxLengthValidator(self._max_length))

    def _validate(self, value: Any) -> Any:
        value = super()._validate(value)

        if self._unique and isinstance(value, Iterable):
            all_set = set()
            for i, v in enumerate(value):
                if v in all_set:
                    e = ValidationError(f'Value "{v}" is not unique.')
                    e.set_index(i)
                    raise e
                else:
                    all_set.add(v)

        return value

    def _cast_to_python(self, value: Any) -> List[Any]:
        if not isinstance(value, list):
            raise ValidationError(f'Value "{value}" must be a list.')

        rl = list()
        for i, v in enumerate(value):
            try:
                cleaned_value = self._element_field.clean(v)
            except ValidationError as e:
                e.set_index(i)
                raise e
            rl.append(cleaned_value)

        return rl

    def _cast_to_json(self, cleaned_value_list: List[Any]) -> List[Any]:
        rl = list()
        for cleaned_value in cleaned_value_list:
            v = self._element_field.to_json(cleaned_value)
            rl.append(v)
        return rl

    def _cast_to_bson(self, cleaned_value_list: List[Any]) -> List[Any]:
        rl = list()
        for cleaned_value in cleaned_value_list:
            v = self._element_field.to_bson(cleaned_value)
            rl.append(v)
        return rl

    def _get_json_type_and_openapi_format(self) -> Tuple[str, str]:
        return JsonType.Array, OpenAPIFormat.Array

    def generate_openapi_object(self) -> dict:
        od = super().generate_openapi_object()

        od["items"] = self._element_field.generate_openapi_object()

        set_nonempty_kv(od, "minLength", self._min_length)
        set_nonempty_kv(od, "maxLength", self._max_length)
        set_nonempty_kv(od, "uniqueItems", self._unique)

        return od

    def generate_meta_object(self) -> dict:
        od = super().generate_meta_object()

        od["element_field_meta"] = self._element_field.generate_meta_object()

        od["min_length"] = self._min_length
        od["max_length"] = self._max_length
        od["unique"] = self._unique

        return od
